from gaphor.UML.classes import (
    classconnect,
    classeseditors,
    classespropertypages,
    copypaste,
    interfaceconnect,
)
from gaphor.UML.classes.association import AssociationItem
from gaphor.UML.classes.dependency import DependencyItem
from gaphor.UML.classes.generalization import GeneralizationItem
from gaphor.UML.classes.implementation import ImplementationItem
from gaphor.UML.classes.interface import Folded, InterfaceItem
from gaphor.UML.classes.klass import ClassItem
from gaphor.UML.classes.package import PackageItem

__all__ = [
    "AssociationItem",
    "DependencyItem",
    "GeneralizationItem",
    "ImplementationItem",
    "InterfaceItem",
    "ClassItem",
    "PackageItem",
]
