
/* Custom styling for tool box */
toolitemgroup > button {
    border-width: 1px 0px 0px 0px;
    border-radius: 0px;
    padding-top: 3px;
    padding-bottom: 3px;
    background-image: none;
    box-shadow: none;
}

toolitemgroup:first-child > button {
    border-width: 0px;
}

#diagrams {
    border-top-width: 0px;
}

diagramview {
    background: white;
}