"""
The diagram package contains items (to be drawn on the diagram), tools
(used for interacting with the diagram) and interfaces (used for adapting the
diagram).
"""

import gi

# import gaphor.diagram.general

gi.require_version("PangoCairo", "1.0")
