# Here, order matters
import gaphor.UML.umloverrides
from gaphor.UML import modelfactory as model
from gaphor.UML.uml import *
from gaphor.UML.umlfmt import format
from gaphor.UML.umllex import parse
